import sys
import os
import torch
import fire
from opendr.engine.datasets import PointCloudsDatasetIterator
from opendr.perception.object_detection_3d import VoxelObjectDetection3DLearner
from opendr.perception.object_detection_3d import (
    KittiDataset,
    LabeledPointCloudsDatasetIterator,
)


def train_pointpillars(device="cuda:0", load=0):
    name = "pointpillars_car"
    config = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "pointpillars",
        "car",
        "xyres_16.proto",
    )
    temp_dir = "./run/models"
    model_path = os.path.join(temp_dir, name)

    subsets_path = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "datasets",
        "kitti_subsets",
    )

    dataset = KittiDataset("/data/sets/opendr_kitti/", subsets_path)

    learner = VoxelObjectDetection3DLearner(
        model_config_path=config, device=device, checkpoint_after_iter=1000, checkpoint_load_iter=load
    )

    learner.fit(
        dataset, model_dir=model_path, verbose=True, evaluate=True,
    )
    learner.save(model_path)


def train_vnn_pointpillars(device="cuda:0", load=0):
    name = "vnn_pointpillars_car"
    config = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "pointpillars",
        "car",
        "vnn_xyres_16.proto",
    )
    temp_dir = "./run/models"
    model_path = os.path.join(temp_dir, name)

    subsets_path = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "datasets",
        "kitti_subsets",
    )

    dataset = KittiDataset("/data/sets/opendr_kitti/", subsets_path)

    learner = VoxelObjectDetection3DLearner(
        model_config_path=config, device=device, checkpoint_after_iter=1000, checkpoint_load_iter=load
    )

    learner.fit(
        dataset, model_dir=model_path, verbose=True, evaluate=True,
    )
    learner.save(model_path)


def test_vnn_pointpillars(device="cuda:0", load=0):
    name = "vnn_pointpillars_car"
    config = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "pointpillars",
        "car",
        "vnn_xyres_16.proto",
    )
    temp_dir = "./run/models"
    model_path = os.path.join(temp_dir, name)

    subsets_path = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "datasets",
        "kitti_subsets",
    )

    dataset = KittiDataset("/data/sets/opendr_kitti/", subsets_path)

    learner = VoxelObjectDetection3DLearner(
        model_config_path=config, device=device, checkpoint_after_iter=1000, checkpoint_load_iter=load
    )
    learner.load(model_path)
    result = learner.eval(dataset, verbose=True)
    return result


def train_tanet(device="cuda:0", load=0):
    name = "tanet_car"
    config = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "tanet",
        "car",
        "xyres_16.proto",
    )
    temp_dir = "./run/models"
    model_path = os.path.join(temp_dir, name)

    subsets_path = os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "datasets",
        "kitti_subsets",
    )

    dataset = KittiDataset("/data/sets/opendr_kitti/", subsets_path)

    learner = VoxelObjectDetection3DLearner(
        model_config_path=config, device=device, checkpoint_after_iter=1000, checkpoint_load_iter=load,
    )

    learner.fit(
        dataset, model_dir=model_path, verbose=True, evaluate=True,
    )
    learner.save(model_path)


if __name__ == "__main__":
    fire.Fire()
